import { Persistable, CrudRepository, PageRequest, Page } from '@apaq/leap-data-core';

declare enum Parameter {
    ON_OFF = 1,
    SPEED = 2,
    BOOT_MODE = 6,
    TIMER_MODE = 7,
    TIMER_COUNT_DOWN = 8,
    HUMIDITY_SENSOR_ACTIVATION = 15,
    RELAY_SENSOR_ACTIVIATION = 20,
    VOLTAGE_SENSOR_ACTIVATION = 22,
    HUMIDITY_THRESHOLD = 25,
    CURRENT_RTC_BATTERY_VOLTAGE = 36,
    CURRENT_HUMIDITY = 37,
    CURRENT_VOLTAGE_SENSOR_STATE = 45,
    CURRENT_RELAY_SENSOR_STATE = 50,
    MANUAL_SPEED = 68,
    FAN1RPM = 74,
    FAN2RPM = 75,
    FILTER_TIMER = 100,
    RESET_FILTER_TIMER = 101,
    BOOST_MODE_DEACTIVATION_DELAY = 102,
    RTC_TIME = 111,
    RTC_CALENDAR = 112,
    WEEKLY_SCHEDULE = 114,
    SCHEDULE_SETUP = 119,
    SEARCH = 124,
    PASSWORD = 125,
    MACHINE_HOURS = 126,
    RESET_ALARMS = 128,
    READ_ALARM = 131,
    CLOUD_SERVER_OPERATION_PERMISSION = 133,
    READ_FIRMWARE_VERSION = 134,
    RESTORE_FACTORY_SETTINGS = 135,
    FILTER_ALARM = 136,
    WIFI_MODE = 148,
    WIFI_NAME = 149,
    WIFI_PASSWORD = 150,
    WIFI_ENCRYPTION = 153,
    WIFI_CHANNEL = 154,
    WIFI_DHCP = 155,
    IP_ADDRESS = 156,
    SUBNET_MASK = 157,
    GATEWAY = 158,
    CURRENT_IP_ADDRESS = 163,
    VENTILATION_MODE = 183,
    UNIT_TYPE = 185
}
declare namespace Parameter {
    function getSize(parameter: Parameter): number;
}

interface DataEntry {
    parameter: Parameter;
    value?: Uint8Array;
}
declare namespace DataEntry {
    function of(parameter: Parameter, value?: number): {
        parameter: Parameter;
        value: Uint8Array;
    };
}

declare enum FunctionType {
    READ = 1,
    WRITE = 2,
    WRITEREAD = 3,
    INCREAD = 4,
    DECREAD = 5,
    RESPONSE = 6
}

declare class Packet {
    private _deviceId;
    private _password;
    private _functionType;
    private _dataEntries;
    constructor(deviceId: string, password: string, functionType: FunctionType, dataEntries: DataEntry[]);
    get deviceId(): string;
    get password(): string;
    get functionType(): FunctionType;
    get dataEntries(): DataEntry[];
    toBytes(): Uint8Array;
    static fromBytes(bytes: Uint8Array): Packet;
    private static readCredential;
    private writeCredential;
    private static readParameters;
    private static calculateChecksum;
}

interface Response {
    packet: Packet;
    ip: string;
}

interface DeviceAddress {
    id: string;
    ip: string;
}
declare class BlaubergVentoClient {
    timeout: number;
    /**
     * Find devices on the network by emitting a broadcast package and collecting all answering controllers.
     * @returns
     */
    findDevices(): Promise<DeviceAddress[]>;
    /**
     * Sends a packet to a specific controller.
     * @param ip The ip of the controller.
     * @param packet The packet to send.
     * @returns Either a packet or nothing, depending on the packet sent to the controller.
     */
    send(packet: Packet, ip?: string): Promise<Response | void>;
}

/**
 * Directions avaiable for a device.
 * The ONEWAY direction is decided by the dip switch on the device
 */
declare enum Mode {
    ONEWAY = 0,
    TWOWAY = 1,
    IN = 2
}

/**
 * Device speed options available.
 */
declare enum Speed {
    OFF = 0,
    LOW = 1,
    MEDIUM = 2,
    HIGH = 3,
    MANUAL = 255
}

/**
 * A class representing a single Duke One Device
 */
declare class Device implements Persistable<string> {
    id: string;
    password: string;
    speed?: Speed;
    mode?: Mode;
    manualSpeed: number;
    fan1Rpm: number;
    humidity: number;
    filterAlarm: boolean;
    filterTime: number;
    on: boolean;
    firmwareVersion: string;
    firmwareDate: Date;
    unitType: number;
    ipAddress: string;
    constructor(id: string, password: string);
    /**
     * @deprecated
     * Use static method instead. (Device.toPacket())
     * @returns The packet for low level com.
     */
    toPacket(): Packet;
    static toPacket(device: Device): Packet;
    static fromPacket(packet: Packet): Device;
    private static applyParameter;
}

declare class BlaubergVentoResource implements CrudRepository<Device, string> {
    private client;
    private _ipMap;
    constructor();
    findAll(pageable?: PageRequest): Promise<Page<Device>>;
    findById(id: string): Promise<Device>;
    save(entity: Device): Promise<Device>;
    saveAll(entities: Device[]): Promise<Device[]>;
    deleteById(id: string): Promise<void>;
    deleteAllById(ids: string[]): Promise<void>;
    delete(entity: Device): Promise<void>;
    deleteAll(entities: Device[]): Promise<void>;
    private resolveDevice;
    private resolveIpMap;
}

export { BlaubergVentoClient, BlaubergVentoResource, DataEntry, Device, DeviceAddress, FunctionType, Mode, Packet, Parameter, Response, Speed };
